#' Create the DEGURBA grid cell classification of rural cells
#'
#' @description
#' The Degree of Urbanisation identifies rural cells as all cells not belonging to an urban centre or urban cluster.
#'
#' For more information about the Degree of Urbanisation methodology, see the [methodological manual](https://ec.europa.eu/eurostat/statistics-explained/index.php?title=Applying_the_degree_of_urbanisation_manual), [GHSL Data Package 2022](https://ghsl.jrc.ec.europa.eu/documents/GHSL_Data_Package_2022.pdf) and [GHSL Data Package 2023](https://ghsl.jrc.ec.europa.eu/documents/GHSL_Data_Package_2023.pdf).
#' @param data path to the directory with the data, or named list with the data as returned by function [DoU_preprocess_grid()]
#' @param classification SpatRaster. A grid with the classification of urban centres and urban clusters to which the classification of rural cells will be added. Note that the grid will be adapted in-place.
#' @param value integer. Value assigned to rural cells in the resulting grid
#' @return SpatRaster with the grid cell classification of rural cells
#' @examples
#' data_belgium <- DoU_load_grid_data_belgium()
#' classification <- DoU_classify_grid_urban_centres(data_belgium)
#' classification <- DoU_classify_grid_urban_clusters(data_belgium, classification = classification)
#' classification <- DoU_classify_grid_rural(data_belgium, classification = classification)
#' DoU_plot_grid(classification)
#' @export
DoU_classify_grid_rural <- function(data, classification, value = 1) {
  # read data
  if (is.character(data)) {
    data <- DoU_preprocess_grid(data)
  }

  # check if the data is valid
  if (
    !all(
      c("pop", "land", "pop_per_land", "built_per_land", "built") %in%
        names(data)
    )
  ) {
    stop(
      "Invalid argument: The data should contain a named list of pop, land, built, pop_per_land and built_per_land as generated by the function DoU_preprocess_grid."
    )
  }

  # all other cells are rural cells
  terra::set.values(classification, which(is.na(classification[])), value)

  # mask non-valid mollweide cells
  classification <- mask_mollweide(classification)

  names(classification) <- c("layer")
  return(classification)
}

#' Create the DEGURBA grid cell classification of rural cells
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' `classify_grid_rural()` has been renamed to `DoU_classify_grid_rural()` to create a more consistent API and to better indicate that this function is specifically designed to classify rural cells in the context of the DEGURBA classification.
#' @param data path to the directory with the data, or named list with the data as returned by function [DoU_preprocess_grid()]
#' @param classification SpatRaster. A grid with the classification of urban centres and urban clusters to which the classification of rural cells will be added. Note that the grid will be adapted in-place.
#' @param value integer. Value assigned to rural cells in the resulting grid
#' @return SpatRaster with the grid cell classification of rural cells
#' @keywords internal
#' @export
classify_grid_rural <- function(data, classification, value = 1) {
  return(DoU_classify_grid_rural(data, classification, value))
}
